""" Vm Image Kit

Usage:
    vik build --distro=DISTRO --release=RELEASE
        [--arch=ARCH | --target=PLATFORM | --output=FILE]
    vik clean
    vik [-hv]

Commands:
    build  build vm image by distribution and corresponding release.
    claen  clean cached installtion medias and cached output images.

Options:
    -a ARCH, --arch=ARCH            OS architectures. [default: amd64]
    -d DISTRO, --distro=DISTRO      OS distribution.
    -r RELEASE, --release=RELEASE   OS release number.
    -t PLATFORM, --target=PLATFORM  Target platform. [default: virtualbox]
    -o FILE, --output=FILE          Output image box.
    -h, --help                      Show this help.
    -v, --version                   Show version.

Examples:
    Create debian release 7.4.0 virtualbox image for vagrant use.

        vik build -d debian -r 7.4.0

    Clean internal cached files.

        vik clean

"""
import os
import sys
import shutil

from vik import check
from vik.adapter import Adapter

""" vik version """
__version__ = '0.0.1b1'

""" vik use packer for image creation."""
IMAGE_TOOL_CMD = 'packer'


def main(args):
    """ Main entry point.

    Parse arguments from docopt dictionary and Check the requirements for vik
    processing.

    Args:
        args: argument dictionary from docopt processed.
    Returns:

    Raises:

    """

    if args['build']:
        # Check if current OS type is suitable for vik processing.
        # If not suitable, raise os not support exception and exit the program.
        if not check.is_os_compatiable():
            sys.exit('system platform is not supported.')

        # Check encapsulates image making tool exists.
        # If not exist, raise tool not found exception and exit the program.
        if not check.has_tool(IMAGE_TOOL_CMD):
            sys.exit('vik reuqire "packer" tool to be installed.')

        print('Requirement test..'.ljust(40), '[DONE]')

        sub_keys = ('--distro', '--release', '--arch', '--target', '--output')
        strip = lambda dict, keys: {key[2:]: dict[key] for key in keys}

        image = Adapter(strip(args, sub_keys))

        image.build()

    elif args['clean']:
        clean()


def clean():
    """ Clean cahced files.
    """

    packer_cache = os.path.join(os.path.dirname(__file__),
                                'vmconfigs',
                                'packer_cache')
    if os.path.isdir(packer_cache):
        shutil.rmtree(packer_cache)

    post_processors = os.path.join(os.path.dirname(__file__),
                                   'vmconfigs',
                                   'post-processors')
    if os.path.isdir(post_processors):
        for root, dirs, files in os.walk(post_processors):
            for file in files:
                os.remove(os.path.join(root, file))
                print('Clean cached file: ', repr(file).ljust(40), '[DONE]')

    print('Clean all caches files.')
